Index: main/channel.c
===================================================================
--- main/channel.c	(revision 376623)
+++ main/channel.c	(working copy)
@@ -941,6 +941,21 @@
 	return NULL;
 }
 
+static struct ast_frame *huc_event_cb(struct ast_channel *chan,
+		struct ast_frame *frame,
+		enum ast_framehook_event event,
+		void *data) {
+	if (!frame || event != AST_FRAMEHOOK_EVENT_READ) {
+		return frame;
+	}
+	if (frame->frametype == AST_FRAME_CONTROL && frame->subclass.integer == AST_CONTROL_PVT_CAUSE_CODE) {
+		ast_channel_lock(chan);
+		ast_channel_hangupcause_hash_set(chan, frame->data.ptr, frame->datalen);
+		ast_channel_unlock(chan);
+	}
+	return frame;
+}
+
 static const struct ast_channel_tech null_tech = {
 	.type = "NULL",
 	.description = "Null channel (should not see this)",
@@ -963,6 +978,10 @@
 	struct ast_sched_context *schedctx;
 	struct ast_timer *timer;
 	struct timeval now;
+	struct ast_framehook_interface huc_interface = {
+		.version = AST_FRAMEHOOK_INTERFACE_VERSION,
+		.event_cb = huc_event_cb,
+	};
 
 	/* If shutting down, don't allocate any new channels */
 	if (ast_shutting_down()) {
@@ -1027,6 +1046,10 @@
 		ast_channel_timingfd_set(tmp, ast_timer_fd(ast_channel_timer(tmp)));
 	}
 
+	if (ast_framehook_attach(tmp, &huc_interface) < 0) {
+		return ast_channel_unref(tmp);
+	}
+
 	/*
 	 * This is the last place the channel constructor can fail.
 	 *
@@ -5791,10 +5814,6 @@
 					timeout = 0;		/* trick to force exit from the while() */
 					break;
 
-				case AST_CONTROL_PVT_CAUSE_CODE:
-					ast_channel_hangupcause_hash_set(chan, f->data.ptr, f->datalen);
-					break;
-
 				case AST_CONTROL_PROGRESS:
 					if (oh && oh->connect_on_early_media) {
 						*outstate = f->subclass.integer;
@@ -5803,6 +5822,7 @@
 					}
 					/* Fallthrough */
 				/* Ignore these */
+				case AST_CONTROL_PVT_CAUSE_CODE:
 				case AST_CONTROL_PROCEEDING:
 				case AST_CONTROL_HOLD:
 				case AST_CONTROL_UNHOLD:
